#include <stdlib.h>
#include <stdio.h>

#define ds_wav_c
#define LUA_LIB

#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"

#include <ulib/ulib.h>
#include "mp3_shared.h"
#include <sys/stat.h>
#include <sys/dir.h>

#include "vars.h"

static int luawav_stop(lua_State *L)
{
	int channel = luaL_optnumber(L, 1, -1);
	channel += 2;

	wav_unload((channel <= 1) ? -1 : channel);
	return 0;
}

static int luawav_load_handle(lua_State *L)
{
	char* filename = (char*)luaL_checkstring(L, 1);
	bool errors = (lua_isboolean(L, 2)) ? (bool)lua_toboolean(L, 2) : false;

	struct stat file_status;
    if(stat(filename, &file_status) != 0)
	{
        luaL_error(L, "File '%s' doesn't exist.", filename);
		return 0;
	}

	int error;
	wav_stuffs *handle = wav_load_handle(filename, &error);
	switch(error) // error
	{
		case 1: 
			luaL_error(L, "Failed to load wav '%s': drwav_init_file() failed.", filename);
			return 0;
		case 2:
			luaL_error(L, "Failed to load wav '%s': Could not allocate audio in memory.", filename);
			return 0;
		case 3:
			luaL_error(L, "Failed to load wav '%s': No frames were decoded from file.", filename);
			return 0;
	}
	// no error here, file loaded
	
	lua_pushlightuserdata(L, handle);
	return 1;
}

static int luawav_play_handle(lua_State *L)
{
	wav_stuffs *handle = (wav_stuffs *)lua_touserdata(L, 1);
	int channel = luaL_checknumber(L, 2);
	bool loop = (lua_isboolean(L, 3)) ? (bool)lua_toboolean(L, 3) : false;
	channel += 2;
	assert(L, handle != NULL, "Bad sound handle.");
	assert(L, channel >= 2 && channel <= 15, "You must select a channel from between 0-13.");

	wav_play_handle(handle, channel, loop);
	return 0;
}

static int luawav_set_volume(lua_State *L)
{
	wav_stuffs *handle = lua_touserdata(L, 1);
	u16 volume = (u16)luaL_checknumber(L, 2);
	assert(L, handle != NULL, "Bad sound handle.");

	wav_set_volume(handle, volume);
	return 0;
}

static int luawav_free_handle(lua_State *L)
{
	wav_stuffs *handle = lua_touserdata(L, 1);
	assert(L, handle != NULL, "Bad sound handle.");

	wav_free_handle(handle);
	return 0;
}

static int luawav_get_length(lua_State *L)
{
	wav_stuffs *handle = lua_touserdata(L, 1);
	assert(L, handle != NULL, "Bad sound handle.");
	
	lua_pushnumber(L, wav_get_length(handle));
	return 1;
}

static const luaL_Reg wavlib[] = {
	{"load", luawav_load_handle},
	{"play", luawav_play_handle},
	{"stop", luawav_stop},
	{"free", luawav_free_handle},
	{"setVolume", luawav_set_volume},
	{"length", luawav_get_length},
    {NULL, NULL}
};


LUALIB_API int luaopen_wav (lua_State *L) {
    luaL_register(L, LUA_WAVLIBNAME, wavlib);
    return 1;
}
