/*
    Copyright 2008-2013 Risike, Reylak, MicroLua's developers

    This file is part of MicroLua.

    MicroLua is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MicroLua is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MicroLua.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <nds.h>
#include <filesystem.h>

#include <stdio.h>
#include <stdlib.h>

#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>

#include <ulib/ulib.h>
#include <fat.h>
#include <unistd.h>

#include <dswifi9.h>
#include <netinet/in.h>

#include "constants.h"
#include "efs_lib.h"
#include "mp3_shared.h"

void print_error(const char *text)
{
	while (1)
	{
		ulStartDrawing2D();
		ulDrawTextBox(8,8,256,192,text,0);
		ulEndDrawing();
		ulSyncFrame();
		
		scanKeys();
		if(keysDown()&KEY_START) break;
	}
}

void print_text(int x, int y, const char *text)
{
	ulStartDrawing2D();
	ulDrawString(x, y, text);
	ulEndDrawing();
	ulSyncFrame();
}

int main()
{
	/*
	
	bool mp3 = false;

	consoleDemoInit();  //setup the sub screen for printing
	iprintf("Init LibFat...");
	if(!fatInitDefault())
	{
		iprintf("failed!");
		while(1)
		{
			swiWaitForVBlank();
		}
	}

	iprintf("success!\nInit MP3 Streamer...");
	if(mp3_init() == 0)
	{
		iprintf("failed!\n");
	}
	iprintf("success!\n");

	while(1) {
		//mp3_print();
		if (mp3) mp3_fill_buffer();
		swiWaitForVBlank();

		scanKeys();
		if (keysDown() & KEY_B)
		{
			if (mp3) mp3_stop();
			else
			{
				mp3_play("/lua/scripts/examples/mp3/wisphunt.mp3", 1, 21.08);
			}
			mp3 = !mp3;
		}
		else if (keysDown() & KEY_A)
		{
			if (mp3) mp3_stop();
			else
			{
				mp3_play("/lua/scripts/examples/mp3/fortress.mp3", 1, 11.43);
			}
			mp3 = !mp3;
		}
		else if (keysDown() & KEY_Y)
		{
			if (mp3) mp3_stop();
			else
			{
				mp3_play("/lua/scripts/examples/mp3/greatgigantic.mp3", 1, 0);
			}
			mp3 = !mp3;
		}
		else if (keysDown() & KEY_X)
		{
			if (mp3) mp3_stop();
			else
			{
				mp3_play("/lua/scripts/examples/mp3/test.mp3", 1, 0);
			}
			mp3 = !mp3;
		}
	}

	*/

	mp3_init();

    // Initialization of library
    ulInit(UL_INIT_ALL);
    ulInitGfx();
    ulInitText();

    // Sucks banks C-D + drops at 30 fps
    ulInitDualScreenMode();

    // Banks A-B for textures VRAM
    ulSetTexVramParameters(UL_BANK_A | UL_BANK_B, VRAM_A, 256 << 10);

    // Banks F-G for palettes VRAM
    //ulSetTexPalVramParameters(UL_BANK_F | UL_BANK_G, VRAM_F, 32 << 10); // this line causes a black screen on DraStic and NO$GBA

    // Use bright pink as a transparent color
    ulSetTransparentColor(RGB15(31, 0, 31));

    // Initialization of libfat and EFSLib
    // First we try both, and if it fails (e.g because the binary
    // hasn't been patched) we try FAT alone
	/*print_text(8, 8, "Initializing EFS");
    if (!EFS_Init(EFS_ONLY, NULL)) {
		print_text(8, 8, "Failed to initialize EFS, using libfat");
        if (!fatInitDefault()) {
            print_error("Failed to initialize Embedded File System\nand FAT together, and FAT alone");
            return 0;
        }
    }*/
	if (!NITRO && !fatInitDefault())
	{
		print_error("Failed to init FAT");
		return 1;
	}
	else if (NITRO)
	{
		if (!nitroFSInit(NULL))
		{
			print_error("Failed to init NitroFS");
			return 1;
		}
	}

    timerStart(TIMER_ID, ClockDivider_1024, 0, NULL);

    struct lua_State *l = luaL_newstate();
    if (!l) {
        print_error("\n\n\tFailed to create a Lua state - Push A to Exit\n");
        return 0;
    }

    luaL_openlibs(l);
    uLua_pushConstants(l);
	
	
    // First we try to load from fat:
    if (luaL_loadfile(l, ULUA_BOOT_FULLPATH)) {
		print_text(8, 8, "loadfile 1");
        if (luaL_loadfile(l, ULUA_LIBS"libs.lua")) {
			char text[256];
			sprintf(text,"Couldn't load %s\n", ULUA_BOOT_FULLPATH);
			print_error(text);
			return 0;
        }
    }

    if(lua_pcall(l,0,0,0)) {
        print_error(lua_tostring(l, -1));
        lua_pop(l, 1);
        return 0;
    }

    lua_close(l);

    return 0;
}
