bird = {}
local snd_flap, snd_hit, snd_squish

function bird.init()
	snd_flap = WAV.load("data/audio/wing.wav")
	snd_hit = WAV.load("data/audio/hit.wav")
	snd_squish = WAV.load("data/audio/point.wav")
	bird.img = Image.load("data/sprites/yellowbird.png", VRAM)
	bird.bloodsplat = Image.load("data/sprites/bloodsplatter.png", RAM)
	bird.restart()
end


function bird.addNewBird()
	table.insert(bird.entities, {
		x = -Image.width(bird.img),
		y = 32+rand(80),
		hspeed = 1,
		vspeed = -4 + rand(6),
		anger = 0,
		frame = 0,
		frame_tick = 0,
		frame_add = 1,
		
		flap = function(self)
			if bird.flaptick <= 0 then
				WAV.play(snd_flap, 0)
				bird.flaptick = math.floor(WAV.length(snd_flap)*60)
			end
			self.vspeed = -4
		end,
		
		doFrameTick = function(self)
			self.frame_tick = self.frame_tick + 1
			if self.frame_tick > 6 then
				self.frame_tick = 0
				local newframe = self.frame + self.frame_add
				if newframe < 0 or newframe > 2 then
					self.frame_add = -self.frame_add
					newframe = self.frame + self.frame_add
				end
				self.frame = newframe
			end
		end
	})
end


function bird.restart()
	bird.entities = {}
	bird.chance = 9900
	bird.flaptick = 0
	
	bird.bloodsplat_tick = 0
	bird.bloodsplat_frame = 0
	
	local birds = {"yellow", "blue", "red"}
	Image.destroy(bird.img)
	bird.img = nil
	collectgarbage()
	bird.img = Image.load("data/sprites/"..birds[rand(3) + 1].."bird.png", VRAM)
end


function bird.tick()
	if rand(10000) >= bird.chance then
		bird.addNewBird()
	end

	local i = 1
	local squished = false
	local hit = false

	while i < #bird.entities do
		local ent = bird.entities[i]

		ent:doFrameTick()

		if pipe.squishing then
			if ent.x >= pipe.x and ent.x < pipe.x + pipe.frame_width then -- squished
				if bird.chance > 9800 then bird.chance = bird.chance - 10 end
				local bonus = 4*ent.anger
				score = score + (bonus > 0 and bonus or 1)
				squished = true

				table.remove(bird.entities, i)
				i = i-1
				goto continue
			elseif ent.x >= pipe.x-8 and ent.x < pipe.x then -- bounce back, anger up
				hit = true
				ent.anger = ent.anger + 1
				ent.hspeed = -6 - (ent.anger/2)
			end
		end

		--if ent.x > 256 then
			--table.remove(bird.entities, i)
			--i = i-1
			--goto continue
		if ent.x >= pipe.x + pipe.frame_width + 20 then -- game over
			menuState = 2
			MP3.stop()
			WAV.play(snd_whoosh, 3)
		elseif ent.y >= 96 and ent.vspeed > 0 then
			ent:flap()
		end

		if ent.hspeed < 1+(ent.anger/4) then ent.hspeed = ent.hspeed + 0.25 end
		ent.vspeed = ent.vspeed + 0.25
		ent.x = ent.x + ent.hspeed
		ent.y = ent.y + ent.vspeed

		::continue::
		i = i+1
	end
	
	if squished then
		WAV.play(snd_squish, 1)
		bird.bloodsplat_frame = 1
	end
	if hit then WAV.play(snd_hit, 2) end

	if bird.flaptick > 0 then bird.flaptick = bird.flaptick - 1 end
	if bird.bloodsplat_frame > 0 then
		bird.bloodsplat_tick = bird.bloodsplat_tick + 1

		if bird.bloodsplat_tick > 5 then
			bird.bloodsplat_tick = 0
			bird.bloodsplat_frame = bird.bloodsplat_frame + 1

			if bird.bloodsplat_frame > 3 then
				bird.bloodsplat_frame = 0
			end
		end
	end
end


function bird.draw()
	for i, ent in pairs(bird.entities) do
		local angerColor = clamp(31-(ent.anger*4), 0, 31)
		Image.setTint(bird.img, Color.new(31, angerColor, angerColor))
		screen.blit(SCREEN_UP, ent.x, ent.y, bird.img, ent.frame * (Image.width(bird.img)/3), 0, Image.width(bird.img)/3, Image.height(bird.img))
	end
	
	if bird.bloodsplat_frame > 0 then
		local x = 			 bird.bloodsplat_frame % 2
		local y = math.floor(bird.bloodsplat_frame / 2)
		screen.blit(SCREEN_UP, 64, 48, bird.bloodsplat, x*128, y*96, 128, 96)
	end
end


function bird.destroy()
	WAV.stop(0)
	WAV.stop(1)
	WAV.stop(2)
	WAV.free(snd_flap)
	WAV.free(snd_hit)
	WAV.free(snd_squish)
	bird.restart()
	Image.destroy(bird.img)
	Image.destroy(bird.bloodsplat)
	bird.img = nil
	bird.bloodsplat = nil
	snd_flap = nil
	snd_hit = nil
	snd_squish = nil
	snd_point = nil
	collectgarbage()
end